/******************************************************************************
 * Wiznet W5100/W3150 Driver.
 * Copyright Andreas Kaiser 2007.
 ******************************************************************************/

#ifndef W5100_COMMON_H_
#define W5100_COMMON_H_

#include "w5100-defs.h"
#include "Util/rprintf.h"

#if W5DEBUGLVL
  #define W5DEBUG(lvl,s,...) do{ if (lvl <= W5DEBUGLVL) rprintf(s, ## __VA_ARGS__); }while(0)
  #define W5PADDR(lvl,s,a)   do{ if (lvl <= W5DEBUGLVL) tcpip_paddr(s,a); }while(0)
#else
  #define W5DEBUG(lvl,s,...)
  #define W5PADDR(lvl,s,a)
#endif

#define W5STATUS(status)	(-0x100-(status))

#ifdef __cplusplus
extern "C" {
#endif

    extern uint16_t	w5port;		// default port number

    extern struct W5Socket_s {
	uint16_t	rbase, rsize;	// Rx buffer base address, size
	uint16_t	tbase, tsize;	// Tx buffer base address, size
	uint8_t		status;
    } w5so[4];

    /*
     * w5100-low.
     */
    void	w5init     (void);
    uint32_t	w5regread  (unsigned addr, unsigned size);
    void	w5regwrite (unsigned addr, uint32_t data, unsigned size);
    void	w5blkread  (unsigned addr, void *data, unsigned size);
    void	w5blkwrite (unsigned addr, const void *data, unsigned size);

    /*
     * w5100-base.
     */
    
    int		w5status  (unsigned so);
    void	w5command (unsigned so, unsigned cmd);
    void	w5wait    (unsigned so, unsigned cmd);

    void	w5interrupt (void);

#ifdef __cplusplus
}
#endif

#endif //W5100_COMMON_H_
