/******************************************************************************
 * Wiznet W5100/W3150 Driver.
 * Copyright Andreas Kaiser 2007.
 ******************************************************************************/

#ifndef W5100_DEFS_H_
#define W5100_DEFS_H_

#define W5MR		0x0000		// 1b - mode register
#define W5GAR		0x0001		// 4b - gateway address
#define W5SUBR		0x0005		// 4b - subnet mask
#define W5SHAR		0x0009		// 4b - hardware address
#define W5SIPR		0x000F		// 4b - IP address
#define W5IR		0x0015		// 1b - interrupt
#define W5IMR		0x0016		// 1b - interrupt mask
#define W5RTR		0x0017		// 2b - retry time
#define W5RCR		0x0019		// 1b - retry count (n*100s)
#define W5RMSR		0x001A		// 1b - Rx memory size
#define W5TMSR		0x001B		// 1b - Tx memory size
#define W5PATR		0x001C		// 2b - PPPoE authentication type
#define W5PTIMER	0x0028		// 1b - PPP LCP request timer (n*25ms)
#define W5PMAGIC	0x0029		// 1b - PPP LCP magic number
#define W5UIPR		0x002A		// 4b - unreachable IP address
#define W5UPORT		0x002E		// 2b - unreachable port

// socket registers
#define W5sR(r,n)	((r)+(n)*0x100)
#define W5sMR(n)	W5sR(0x0400,n)	// 1b - mode
#define W5sCR(n)	W5sR(0x0401,n)	// 1b - command
#define W5sIR(n)	W5sR(0x0402,n)	// 1b - interrupt
#define W5sSR(n)	W5sR(0x0403,n)	// 1b - status
#define W5sSPORT(n)	W5sR(0x0404,n)	// 2b - source port
#define W5sDHAR(n)	W5sR(0x0406,n)	// 6b - destination hardware address
#define W5sDIPR(n)	W5sR(0x040C,n)	// 4b - destination IP address
#define W5sDPORT(n)	W5sR(0x0410,n)	// 2b - destination port
#define W5sMSSR(n)	W5sR(0x0412,n)	// 2b - maximum segment size
#define W5sPROTO(n)	W5sR(0x0414,n)	// 1b - protocol in raw mode
#define W5sTOS(n)	W5sR(0x0415,n)	// 1b - TOS
#define W5sTTL(n)	W5sR(0x0416,n)	// 1b - TTL
#define W5sTXFSR(n)	W5sR(0x0420,n)	// 2b - Tx free size
#define W5sTXRD(n)	W5sR(0x0422,n)	// 2b - Tx read pointer
#define W5sTXWR(n)	W5sR(0x0424,n)	// 2b - Tx write pointer (not immediately echoed)
#define W5sRXRSR(n)	W5sR(0x0426,n)	// 2b - Rx received size
#define W5sRXRD(n)	W5sR(0x0428,n)	// 2b - Rx read pointer

// W5MR: mode register
#define W5MR_SWReset	(1<<7)		// software reset (automatically cleared)
#define W5MR_PB		(1<<4)		// ping block
#define W5MR_PPPoE	(1<<3)		// PPPoE mode
#define W5MR_AI		(1<<1)		// auto-increment address (indirect bus i/f)
#define W5MR_IND	(1<<0)		// indirect bus i/f mode

// W5IR,W5IMR: interrupt cause register, interrupt mask register
#define W5IR_Conflict	(1<<7)		// IP conflict detected, cleared by writing 1
#define W5IR_Unreach	(1<<6)		// ICMP unreachable received, cleared by writing 1
#define W5IR_PPPoE	(1<<5)		// PPPoE connection close, cleared by writing 1
#define W5IR_S3		(1<<3)		// socket 3 interrupt, cleared automatically
#define W5IR_S2		(1<<2)		// socket 2 interrupt, cleared automatically
#define W5IR_S1		(1<<1)		// socket 1 interrupt, cleared automatically
#define W5IR_S0		(1<<0)		// socket 0 interrupt, cleared automatically

// W5RMSR,W5TMSR: memory size
#define W5MSR_1K(n)	(0<<2*(n))	// 1KB
#define W5MSR_2K(n)	(1<<2*(n))	// 2KB
#define W5MSR_4K(n)	(2<<2*(n))	// 4KB
#define W5MSR_8K(n)	(3<<2*(n))	// 8KB

#define W5PATR_PAP	0xC023		// PPPoE authentication by PAP
#define W5PATR_CHAP	0xC223		// PPPoE authentication by CHAP

// W5sMR: socket mode
#define W5sMR_Multi	(1<<7)		// UDP: multicasting
#define W5sMR_ND	(1<<6)		// TCP: no delayed ack
#define W5sMR_MC	(1<<6)		// UDP: 0=IGMPv2, 1=IGMPv1
#define W5sMR_Protocol	0x0F		// protocol
#define W5sMR_Closed	0		// Closed
#define W5sMR_TCP	1		// TCP
#define W5sMR_UDP	2		// UDP
#define W5sMR_IPRAW	3		// IP raw IP
#define W5sMR_MACRAW	4		// MAC war
#define W5sMR_PPPoE	5		// PPPoE

// W5sCR: socket command, automatically cleared
#define W5sCR_Open	0x01		// initialize socket
#define W5sCR_Listen	0x02		// TCP: wait for connection request
#define W5sCR_Connect	0x04		// TCP: connection to server
#define W5sCR_Discon	0x08		// TCP: disconnect from server
#define W5sCR_Close	0x10		// close socket (TCP: no connection termination)
#define W5sCR_Send	0x20		// send data
#define W5sCR_SendMAC	0x21		// UDP: send frame using socket DHAR
#define W5sCR_SendKeep	0x22		// TCP: send keepalive
#define W5sCR_Recv	0x40		// receive data

// W5sIR: socket interrupt, cleared by writing 1
#define W5sIR_Send	(1<<4)		// send operation completed
#define W5sIR_Timeout	(1<<3)		// timeout during connectino establishment or data transmission
#define W5sIR_Recv	(1<<2)		// have data in receive buffer
#define W5sIR_Discon	(1<<1)		// connection termination requested or finished
#define W5sIR_Con	(1<<0)		// connection established

// W5sSR: socket status
#define W5sSR_Closed		0x00	// socket closed
#define W5sSR_SockInit		0x13	// TCP: opened, waiting for listen/connect
#define W5sSR_SockListen	0x14	// TCP: listening, becomes established when connected
#define W5sSR_Established	0x17	// TCP: connection established
#define W5sSR_CloseWait		0x1C	// TCP: connection termination request received from peer
#define W5sSR_UDP		0x22	// UDP: opened
#define W5sSR_IPRAW		0x32	// IPRAW: opened
#define W5sSR_MACRAW		0x42	// MACRAW: opened
#define W5sSR_PPPoE		0x5F	// PPPoE: opened
// transient status:
#define W5sSR_SynSent		0x15	// TCP: while connecting
#define W5sSR_SynRecv		0x16	// TCP: while connecting
#define W5sSR_FinWait		0x18	// TCP: while disconnecting
#define W5sSR_Closing		0x1A	// TCP: while disconnecting
#define W5sSR_TimeWait		0x1B	// TCP: while disconnecting
#define W5sSR_LastAck		0x1D	// TCP: while disconnecting
#define W5sSR_ARP		0x01	// in ARP requests
#define W5sSR_ARP1		0x11	// in ARP requests
#define W5sSR_ARP2		0x21	// in ARP requests
#define W5sSR_ARP3		0x31	// in ARP requests

#define W5TXBUF			0x4000
#define W5TXBUF_END		0x6000
#define W5RXBUF			0x6000
#define W5RXBUF_END		0x8000

#endif //W5100_DEFS_H_
