/******************************************************************************
 * Wiznet W5100/W3150 Driver.
 * Copyright Andreas Kaiser 2007.
 ******************************************************************************/

#include "config.h"
#include "common.h"
#include "w5100.h"
#include "w5100-common.h"

#define W5SPI		SPI(W5SPIno)
#define initCS()	GPIO(W5CSport).dir |= (1<<(W5CSpin))
#define enableCS()	GPIO(W5CSport).clr = (1<<(W5CSpin))
#define disableCS()	GPIO(W5CSport).set = (1<<(W5CSpin))

#ifdef W5EINTno
  #define INTINIT	vic.vector(VIC_EINT(W5EINTno), w5irq, Vic::Low)
  #define INTDIS	bool enabled = vic.enabled(VIC_EINT(W5EINTno)); vic.disable(VIC_EINT(W5EINTno))
  #define INTREST	if (enabled) vic.enable(VIC_EINT(W5EINTno))

  static void
  w5irq(void)
  {
      w5interrupt();
      EXTINT = 1 << W5EINTno;
  }
#else
  #define INTINIT
  #define INTDIS
  #define INTREST
#endif

void
w5init(void)
{
    W5DEBUG(1, "w5init\n");
    INTINIT;
    initCS();
    W5SPI.spcr  = SPCR_MASTER;		// master, mode=0, MSB first, no interrupt
    W5SPI.spccr = 8;			// max clock (7,5MHz @ PCLK=60MHZ)
}

static inline unsigned
spi(unsigned val)
{
    W5SPI.spdr = val;
    while (!(W5SPI.spsr & SPSR_SPIF))
	;
    return W5SPI.spdr;
}

uint32_t
w5read(unsigned addr)
{
    enableCS();
    spi(0x0F);
    spi(addr >> 8);
    spi(addr);
    unsigned val = spi(0x00);
    disableCS();
    return val;
}

void
w5write(unsigned addr, unsigned data)
{
    enableCS();
    spi(0xF0);
    spi(addr >> 8);
    spi(addr);
    spi(data);
    disableCS();
}

uint32_t
w5regread(unsigned addr, unsigned size)
{
    INTDIS;
    W5DEBUG(6, "\n<RD@%04X", addr);
    uint32_t data = 0;
    do {
	data = data << 8 | w5read(addr++);
	W5DEBUG(6, " %02X", data & 0xFF);
    } while (--size != 0);
    W5DEBUG(6, ">\n");
    INTREST;
    return data;
}

void
w5regwrite(unsigned addr, uint32_t data, unsigned size)
{
    INTDIS;
    W5DEBUG(5, "\n<WR@%04X", addr);
    int n = size * 8;
    do {
	n -= 8;
	W5DEBUG(5, " %02X", (data >> n) & 0xFF);
	w5write(addr++, data >> n);
    } while (n);
    W5DEBUG(5, ">\n");
    INTREST;
}

void
w5blkwrite(unsigned addr, const void *data, unsigned size)
{
    INTDIS;
    const uint8_t *p = (const uint8_t *)data;
    W5DEBUG(5, "WR: @%04X #%d\n", addr, size);
    do {
	w5write(addr++, *p++);
    } while (--size);
    INTREST;
}

void
w5blkread(unsigned addr, void *data, unsigned size)
{
    INTDIS;
    uint8_t *p = (uint8_t *)data;
    W5DEBUG(5, "RD: @%04X #%d\n", addr, size);
    do {
	*p++ = w5read(addr++);
    } while (--size);
    INTREST;
}
