/******************************************************************************
 * Wiznet W5100/W3150 Driver.
 * Copyright Andreas Kaiser 2007.
 ******************************************************************************/

#include "config.h"
#include "common.h"
#include "w5100.h"
#include "w5100-common.h"

int
tcp_open(unsigned so, unsigned localport)
{
    W5DEBUG(1, "tcpip:open #%u localport=%u\n", so, localport);
    tcpip_local(so, localport);
    return tcpip_open(so, W5sMR_TCP);
}

int
tcp_connect(unsigned so, uint32_t remoteaddr, unsigned remoteport)
{
    W5DEBUG(1, "tcp:connect #%u", so);
    W5PADDR(1, " remote=", remoteaddr);
    W5DEBUG(1, ", port=%u\n", remoteport);
    w5regwrite(W5sDIPR(so), remoteaddr, 4);
    w5regwrite(W5sDPORT(so), remoteport, 2);
    w5command(so, W5sCR_Connect);
    return tcp_established(so);
}

int
tcp_listen(unsigned so)
{
    W5DEBUG(1, "tcp:listen #%u\n", so);
    w5command(so, W5sCR_Listen);
    return 0;
}

int
tcp_established(unsigned so)
{
    unsigned status = w5status(so), rc = status;
    W5DEBUG(2, "tcp:established #%u status=%s\n", so, tcpip_error(status));
    if (status == W5sSR_Established)
	return 0;
    if ((status & 0x0F) == W5sSR_ARP || status >= W5sSR_SockInit && status < W5sSR_Established)
	return status;
    return W5STATUS(rc);
}

uint32_t
tcp_remote_addr(unsigned so)
{
    return w5regread(W5sDIPR(so), 4);
}

unsigned
tcp_remote_port(unsigned so)
{
    return w5regread(W5sDPORT(so), 2);
}

int
tcp_cancel(unsigned so)
{
    W5DEBUG(1, "tcp:cancel #%u\n", so);
    return tcpip_close(so);
}

int
tcp_close(unsigned so)
{
    W5DEBUG(1, "tcp:close #%u\n", so);
    w5regwrite(W5sCR(so), W5sCR_Discon, 1);
    return tcpip_closed(so);
}

//-----------------------------------------------------------------------------

int
tcp_send_avail(unsigned so)
{
    unsigned status = w5status(so);
    if (status != W5sSR_Established) {
	W5DEBUG(1, "tcp:send_avail #%u status=%s\n", so, tcpip_error(status));
	return W5STATUS(status);
    }
    return tcpip_write_avail(so);
}

int
tcp_send(unsigned so, const void *data, int size)
{
    W5DEBUG(1, "tcpip:send #%u size=%d\n", so, size);

    int nb = tcp_send_avail(so);
    if (nb <= 0)
	return nb;
    if (nb > size)
	nb = size;

    unsigned ptr = w5regread(W5sTXWR(so), 2);
    W5DEBUG(2, "tcpip:send #%u wr=%04X nb=%u\n", so, ptr, nb);
    tcpip_write(so, ptr, (uint8_t *)data, nb);
    w5regwrite(W5sTXWR(so), ptr + nb, 2);
    w5command(so, W5sCR_Send);
    return nb;
}

//-----------------------------------------------------------------------------

int
tcp_recv_avail(unsigned so)
{
    unsigned status = w5status(so);
    if (status != W5sSR_Established) {
	W5DEBUG(1, "tcp:recv_avail #%u status=%s\n", so, tcpip_error(status));
	return W5STATUS(status);
    }
    return tcpip_read_avail(so);
}

int
tcp_recv(unsigned so, void *data, int size)
{
    W5DEBUG(1, "tcp:recv #%u size=%d\n", so, size);
    int nb = tcp_recv_avail(so);
    if (nb < 0)
	return nb;
    if (nb > size)
	nb = size;

    if (nb) {
        unsigned ptr = w5regread(W5sRXRD(so), 2);
	W5DEBUG(2, "tcp:recv #%u rd=%04X nb=%u\n", so, ptr, nb);
	tcpip_read(so, ptr, (uint8_t *)data, nb);
    	w5regwrite(W5sRXRD(so), ptr + nb, 2);
	w5command(so, W5sCR_Recv);
    }

    return nb;
}
