/******************************************************************************
 * Wiznet W5100/W3150 Driver.
 * Copyright Andreas Kaiser 2007.
 ******************************************************************************/

#include "config.h"
#include "common.h"
#include "w5100.h"
#include "w5100-common.h"

static int
udp_opened(unsigned so)
{
    int status = w5status(so), rc = W5STATUS(status);
    if (status != W5sSR_Closed)
	rc = 0;
    W5DEBUG(2, "udp:opened #%u status=%s\n", so, tcpip_error(status));
    return rc;
}

int
udp_open(unsigned so, unsigned localport)
{
    tcpip_local(so, localport);
    return tcpip_open(so, W5sMR_UDP);
}

//-----------------------------------------------------------------------------

int
udp_send_avail(unsigned so)
{
    unsigned status = w5status(so);
    if (status != W5sSR_UDP) {
	W5DEBUG(1, "udp:send_avail #%u status=%s\n", so, tcpip_error(status));
	return W5STATUS(status);
    }
    return tcpip_write_avail(so);
}

int
udp_send(unsigned so, void *data, int size,
	 uint32_t remoteaddr, unsigned remoteport)
{
    W5DEBUG(1, "udp:send #%u size=%d", so, size);
    W5PADDR(1, " remote=", remoteaddr);
    W5DEBUG(1, " port=%u\n", remoteport);

    int nb = tcpip_write_avail(so);
    if (nb < size)
	return 0;

    w5regwrite(W5sDIPR(so), remoteaddr, 4);
    w5regwrite(W5sDPORT(so), remoteport, 2);

    unsigned ptr = w5regread(W5sTXWR(so), 2);
    W5DEBUG(2, "udp:send #%u ptr=%04X nb=%d\n", so, ptr, size);
    tcpip_write(so, ptr, (const uint8_t *)data, size);
    w5regwrite(W5sTXWR(so), ptr + size, 2);
    w5command(so, W5sCR_Send);

    return size;

}

//-----------------------------------------------------------------------------

int
udp_recv_avail(unsigned so)
{
    unsigned status = w5status(so);
    if (status != W5sSR_UDP) {
	W5DEBUG(1, "udp:send_avail #%u status=%s\n", so, tcpip_error(status));
	return W5STATUS(status);
    }
    int nb = tcpip_read_avail(so);
    if (nb < 8)
	return 0;

    uint8_t header[2];
    tcpip_read(so, w5regread(W5sRXRD(so), 2), header, 2);
    int len = tcpip_load16(header + 0);
    W5DEBUG(1, "udp:recv_avail #%u len=%u\n", so, len);
    return len;
}

int
udp_recv(unsigned so, void *data, int size,
	 uint32_t *remoteaddr, uint16_t *remoteport)
{
    W5DEBUG(1, "udp_recv #%u size=%d\n", so, size);

    unsigned status = w5status(so);
    if (status != W5sSR_UDP) {
	W5DEBUG(1, "udp:send_avail #%u status=%s\n", so, tcpip_error(status));
	return W5STATUS(status);
    }
    int nb = tcpip_read_avail(so);
    if (nb < 8)
	return 0;

    unsigned ptr = w5regread(W5sRXRD(so), 2);

    uint8_t header[8];
    tcpip_read(so, ptr, header, 8);
    int len = tcpip_load16(header + 0);
    *remoteaddr  = tcpip_load32(header + 2);
    *remoteport  = tcpip_load16(header + 6);
    W5DEBUG(1, "udp:recv #%u ptr=%04X len=%u", so, ptr, len);
    W5PADDR(1, " addr=", *remoteaddr);
    W5DEBUG(1, " port=%u\n", *remoteport);

    if (len < size)
	size = len;
    else
	W5DEBUG(1, "udp:recv #%u truncated req=%u avail=%u\n", so, size, len);

    W5DEBUG(2, "udp:recv #%u ptr=%04X nb=%u\n", so, ptr+8, size);
    if (size)
	tcpip_read(so, ptr+8, (uint8_t *)data, size);

    w5regwrite(W5sRXRD(so), ptr + len, 2);
    w5command(so, W5sCR_Recv);

    return size;
}
